from django_forest.tests.forest.choice import ChoiceForest
from django_forest.tests.forest.question import QuestionForest
from django_forest.tests.forest.place import PlaceForest

__all__ = ['QuestionForest', 'ChoiceForest', 'PlaceForest']
